#!/bin/bash
usage()
{
  echo "Usage: deploy" 
  exit 2
}

prepare() {
    local sources_list_bak
    local armbian_list_bak
    sources_list_bak=/etc/apt/"$(date +%s)_sources.list.bak"
    armbian_list_bak=/etc/apt/sources.list.d/"$(date +%s)_armbian.list.bak"
    cp /etc/apt/sources.list "$sources_list_bak"
    cp /etc/apt/sources.list.d/armbian.list  "$armbian_list_bak"
    # debian, 有些是 debian
    sed -i'' "s#http[s]\?://deb.debian.org#https://mirrors.tuna.tsinghua.edu.cn#g" /etc/apt/sources.list
    # ubuntu, 有些是 ubuntu
    sed -i'' "s#http[s]\?://ports.ubuntu.com#https://mirrors.tuna.tsinghua.edu.cn/ubuntu-ports#g" /etc/apt/sources.list
    if ! apt update
    then
        cp "$sources_list_bak" /etc/apt/sources.list
        apt update
    fi

    # armbian
    sed -i.bak "s#http[s]\?://apt.armbian.com#https://mirrors.tuna.tsinghua.edu.cn/armbian#g" \
        /etc/apt/sources.list.d/armbian.list
    if ! apt update
    then
        cp "$armbian_list_bak" /etc/apt/sources.list.d/armbian.list
        apt update
    fi

    apt install -y wget 
    apt install -y syslinux-common
    apt install -y curl
    apt install -y sysstat
    apt install -y qrencode
    apt install -y bzip2
    apt install -y nvme-cli
    apt install -y icu-devtools
    apt install -y bc
    apt install -y jq
    apt install -y net-tools
    apt install -y dosfstools
    apt install -y pciutils
    apt install -y lsof
    apt install -y iproute2
    apt install -y miniupnpc
    apt install -y bind9-utils
    apt install -y thin-provisioning-tools lvm2
    apt install -y qemu-utils qemu-user-binfmt
    apt install -y pppoe
    apt install -y bind9-host
    apt install -y dropbear
    apt install -y iputils-ping
    apt install -y file
    apt install -y apt-transport-https
    apt-get install -y ca-certificates gnupg lsb-release

    mkdir -p /usr/share/kbd/consolefonts/
    mkdir -p /usr/share/consolefonts/
    wget https://cdn.linkfog.cn/deploy/fonts/UniCyrExt_8x16.psf.gz -O /usr/share/kbd/consolefonts/UniCyrExt_8x16.psf.gz
    cp /usr/share/kbd/consolefonts/UniCyrExt_8x16.psf.gz -f /usr/share/consolefonts/UniCyrExt_8x16.psf.gz
    echo 'FONT="UniCyrExt_8x16.psf.gz"' >> /etc/default/console-setup

    local docker_version
    docker_version=$(docker --version | awk '{print $3}' | sed 's/,//g' 2>/dev/null)
    if [[ "$docker_version" < "23.00.00" ]]
    then
        install -m 0755 -d /etc/apt/keyrings
        if [[ "$(lsb_release --id -s)" == "Ubuntu" ]]
        then
            curl -fsSL https://download.docker.com/linux/ubuntu/gpg | gpg --yes --dearmor -o /etc/apt/keyrings/docker.gpg
        fi
        if [[ "$(lsb_release --id -s)" == "Debian" ]]
        then
            curl -fsSL https://download.docker.com/linux/debian/gpg | gpg --yes --dearmor -o /etc/apt/keyrings/docker.gpg
        fi
        chmod a+r /etc/apt/keyrings/docker.gpg
    fi

    docker_version=$(docker --version | awk '{print $3}' | sed 's/,//g' 2>/dev/null)
    if [[ "$docker_version" < "23.00.00" ]]
    then
        curl -fsSL https://cdn.linkfog.cn/product/terminal/get_docker.sh | bash -s docker --mirror Aliyun
    fi

    # 如果 aliyun 的源安装 docker-ce 失败用 tuna 的源重试
    if [[ "$docker_version" < "23.00.00" ]]
    then
        systemctl stop docker.socket
        systemctl stop containerd.service
        apt-get remove -y docker docker-engine docker.io containerd runc
        # 先用 tuna 的源尝试安装 docker-ce
        export DOWNLOAD_URL="https://mirrors.tuna.tsinghua.edu.cn/docker-ce"
        curl -fsSL https://cdn.linkfog.cn/product/terminal/get_docker.sh | sh
    fi

    docker_version=$(docker --version | awk '{print $3}' | sed 's/,//g' 2>/dev/null)
    # 如果 aliyun 的源安装 docker-ce 也失败, 用官方源安装
    if [[ "$docker_version" < "23.00.00" ]]
    then
        systemctl stop docker.socket
        systemctl stop containerd.service
        apt-get remove -y docker docker-engine docker.io containerd runc

        if [[ "$(lsb_release --id -s)" == "Ubuntu" ]]
        then
            # Add Docker's official GPG key:
            apt-get install -y ca-certificates curl gnupg lsb-release
            install -m 0755 -d /etc/apt/keyrings
            curl -fsSL https://download.docker.com/linux/ubuntu/gpg | gpg --yes --dearmor -o /etc/apt/keyrings/docker.gpg
            # Add the repository to Apt sources:
            echo \
              "deb [arch="$(dpkg --print-architecture)" signed-by=/etc/apt/keyrings/docker.gpg] https://download.docker.com/linux/ubuntu \
              "$(. /etc/os-release && echo "$VERSION_CODENAME")" stable" | \
                tee /etc/apt/sources.list.d/docker.list > /dev/null
            apt-get update
        fi

        if [[ "$(lsb_release --id -s)" == "Debian" ]]
        then
            # Add Docker's official GPG key:
            apt-get install -y ca-certificates curl gnupg
            install -m 0755 -d /etc/apt/keyrings
            curl -fsSL https://download.docker.com/linux/debian/gpg | gpg --yes --dearmor -o /etc/apt/keyrings/docker.gpg

            # Add the repository to Apt sources:
            echo \
              "deb [arch="$(dpkg --print-architecture)" signed-by=/etc/apt/keyrings/docker.gpg] https://download.docker.com/linux/debian \
              "$(. /etc/os-release && echo "$VERSION_CODENAME")" stable" | \
              tee /etc/apt/sources.list.d/docker.list > /dev/null
            apt-get update
        fi

        apt-get install -y docker-ce docker-ce-cli containerd.io docker-compose-plugin
    fi 
    systemctl enable docker.socket
    systemctl enable docker.service
}

HIDEN_QRCODE="false"
VENDOR="PUTE"
VERSION="1.1.100.57.9"
FSTYPE="lvm"
install_apps_deploy() {
    mkdir -p /writable/
    IMG_DIR=$(mktemp -d /opt/XXXXXX)
    wget https://fae-cdn.linkfog.cn/product/terminal/armbian/PUTE/A03A35/small/1.1.100.57.9/amlogic/lvm/2024-07-02-1719926785-z9fgh5tx/app.img -O "$IMG_DIR/app.img"
    local mount_point
    mount_point=$(mktemp -d)
    mount -n -t squashfs -o rw "$IMG_DIR/app.img" "$mount_point"
    ln -sf "$mount_point/app" /writable/app

    local config_file="/tmp/install_apps.conf"
    cat /writable/app/install_apps.conf | \
        sed '/systemd-cryptsetup/d' | \
        sed '/xcryptsetup/d' | \
        sed '/crypt.conf/d' | \
        sed '/crypttab.temp/d' | \
        sed '/cleanup-mount/d' | \
        sed '/remount-fs-pre/d' | \
        sed '/os-release/d' | \
        sed '/10_linux/d' | \
        sed '/os-release/d' | \
        sed '\#/etc/passwd#d' | \
        sed '\#/etc/shadow#d' | \
        sed '\#/etc/rc.local#d' | \
        sed '\#/etc/rc.local.origin#d' | \
        sed '\#/etc/interfaces#d' | \
        sed '\#/etc/rc.local.web#d' | \
        sed '\#/usr/sbin/armbian-update#d' | \
        sed '\#/usr/sbin/armbian-install#d' | \
        sed '\#/bin/setpasswd.sh#d' | \
        sed '\#/root/.ssh/authorized_keys#d' | \
        sed '\#/etc/dropbear/authorized_keys#d' | \
        sed '/redhat-release/d' | \
        sed '/centos-release/d' > $config_file

    printf "$VERSION" > /writable/.deploy_install
    source /writable/app/install_apps.sh

    install_apps "$FSTYPE" "$config_file"
    umount "$mount_point"
    rm -f "$config_file"

    rm -f /writable/.installation

    if [[ -e /boot/uEnv.txt ]]
    then
        sed -i'.bak' 's/APPEND=root=\(.*\)/APPEND=root=\1 systemd.unified_cgroup_hierarchy=0/g' /boot/uEnv.txt
    fi
    if [[ -e /boot/armbianEnv.txt ]]
    then
        if ! grep -q "extraargs" /boot/armbianEnv.txt
        then
            echo "extraargs=systemd.unified_cgroup_hierarchy=0" >> /boot/armbianEnv.txt
        else
            sed -i'.bak' "s|extraargs=\(.*\)|extraargs=\1 systemd.unified_cgroup_hierarchy=0|g" /boot/armbianEnv.txt
        fi
    fi
    if [[ -e  /boot/extlinux/extlinux.conf ]]
    then
        sed -i'.bak' "s|append \(.*\)|append \1 systemd.unified_cgroup_hierarchy=0|g" /boot/extlinux/extlinux.conf
    fi

    if [[ "$HIDEN_QRCODE" == "true" ]]
    then
        touch /writable/.hiden_qrcode
        issue_file="/writable/.isuse"
        echo "$issue_file" > /writable/.issue_file
    else
        issue_file="/etc/issue"
    fi

    if [[ ! -f /writable/.deviceID ]]
    then
        /usr/bin/create_device_id.sh "$issue_file"
    fi

    mkdir -p /etc/modules-load.d/
    echo 'xt_FULLCONENAT' > /etc/modules-load.d/xt_FULLCONENAT.conf
    sync

    cat /writable/.deviceID; echo
    cat /etc/issue
    sleep 3
}

cleanup() {
    rm -rf "$IMG_DIR/app.img"
    rm -rf "$IMG_DIR/"
    rm -f /writable/app
}

echo "v1.0.5"
prepare
install_apps_deploy
cleanup
if [[ "$1" !=  "1" ]]
then
    systemctl daemon-reload
    if ! systemctl start installer.service
    then
        /usr/bin/diskManager_lvm -s '/' --fstype ext4
    fi
    sleep 5
    systemctl start docker.socket || systemctl start docker.service
    systemctl start agent-new.service
    systemctl start polar-net.service
else
    systemctl reboot
fi
